#!/bin/bash
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8
# ==DIS/AUTOEXEC==
# === 🛠️📦 Build Seal Install ===
# 11_build_installer.sh
# Автоматическая сборка архива и склейка с установщиком
#

echo "Сборка архива и склейка с установщиком"
echo "───────────────────────────────────────────"

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
INSTALLER="$SCRIPT_DIR/rpi_seal_installer.sh"
ARCHIVE="$SCRIPT_DIR/rpi_seal.tar.gz"
OUTPUT="$SCRIPT_DIR/rpi_seal_full_installer.sh"

cd "$SCRIPT_DIR" || exit 1

echo "[*] Создаю архив $ARCHIVE из файлов каталога..."

# Собираем список файлов для архивации (исключая установщик, архив, выходной файл)
INCLUDE_FILES=()
for f in ./*; do
  fname="$(basename "$f")"
  case "$fname" in
    "$(basename "$ARCHIVE")"|"$(basename "$OUTPUT")") continue ;;
#    "$(basename "$INSTALLER")"|"$(basename "$ARCHIVE")"|"$(basename "$OUTPUT")") continue ;;
    *.sh|*.md) INCLUDE_FILES+=("$f") ;;
  esac
done

if [[ ${#INCLUDE_FILES[@]} -eq 0 ]]; then
  echo "❌ Нет файлов для архивации"
  exit 1
fi

tar -czf "$ARCHIVE" "${INCLUDE_FILES[@]}"

# Проверка целостности архива
if ! tar -tzf "$ARCHIVE" &>/dev/null; then
  echo "❌ Ошибка в архиве: возможно, он повреждён"
  exit 1
fi

# Проверка наличия установщика
if [[ ! -f "$INSTALLER" ]]; then
  echo "❌ Файл установщика не найден: $INSTALLER"
  exit 1
fi

# Удаляем старый склеенный файл перед созданием
rm -f "$OUTPUT"

echo "[*] Склеиваю $INSTALLER + $ARCHIVE → $OUTPUT"
cat "$INSTALLER" "$ARCHIVE" > "$OUTPUT"
chmod +x "$OUTPUT"

echo "✅ Готово: $OUTPUT"
