#!/bin/bash
# ==DIS/AUTOEXEC==
# === Import rPi Seal ===
# 14_rpi_seal_prepare_img.sh внедрение rpi_seal
#


echo "Внедрение актуальной сборки rpi_seal в img"
echo "───────────────────────────────────────────"


# Проверка на запуск от root, если нет — перезапуск через sudo
if [[ "$EUID" -ne 0 ]]; then
  exec sudo "$0" "$@"
fi

# Точка выхода для очистки
cleanup() {
  [[ -d "$MOUNT_DIR" && $(mountpoint -q "$MOUNT_DIR") ]] && umount "$MOUNT_DIR"
  [[ -n "$LOOPDEV" && -b "$LOOPDEV" ]] && losetup -d "$LOOPDEV"
}
trap cleanup EXIT

# Путь к локальному каталогу с актуальной версией rpi_seal (тот же каталог, где скрипт)
LOCAL_SEAL_DIR="$(dirname "$0")"

if [[ ! -d "$LOCAL_SEAL_DIR" ]]; then
  echo "[!] Каталог с rpi_seal не найден: $LOCAL_SEAL_DIR"
  exit 1
fi

# --- Выбор устройства / начало блока ---
NAS_PATH="/mnt/backup_nas"
if mountpoint -q "$NAS_PATH"; then
  mapfile -t DEVICES < <(lsblk -nrpo NAME,SIZE,MOUNTPOINT | grep -E "/dev/sd|/dev/nvme" | awk '{print $1":"$2":"$3}')
  DEVICES+=("$NAS_PATH::NAS")
else
  mapfile -t DEVICES < <(lsblk -nrpo NAME,SIZE,MOUNTPOINT | grep -E "/dev/sd|/dev/nvme" | awk '{print $1":"$2":"$3}')
fi

if [[ ${#DEVICES[@]} -eq 0 ]]; then
  echo "❌ Нет доступных устройств."
  exit 1
fi

echo "💾 Выберите диск с образами .img/.img.gz:"
i=1
for dev in "${DEVICES[@]}"; do
  IFS=":" read -r name size mnt <<< "$dev"
  label=""
  [[ "$mnt" == "NAS" ]] && label="🖧 сетевой каталог"
  [[ -z "$mnt" && -z "$label" ]] && label="🔌 не примонтировано"
  echo "  $i - $name $size $mnt $label"
  ((i++))
done

echo "  0 - ❌ Отмена"
read -rp "👉 Ваш выбор: " choice
[[ "$choice" == "0" || -z "$choice" ]] && echo "❎ Отменено." && exit 0
(( choice < 1 || choice > ${#DEVICES[@]} )) && echo "❌ Неверный выбор." && exit 1

SEL_DEV="${DEVICES[$((choice-1))]}"
IFS=":" read -r DEV_NAME DEV_SIZE DEV_MNT <<< "$SEL_DEV"

TEMP_MOUNTED=false
if [[ "$DEV_NAME" == "$NAS_PATH" ]]; then
  TARGET_DIR="$NAS_PATH"
else
  MOUNTED_PATH=$(lsblk -nrpo NAME,MOUNTPOINT "$DEV_NAME" | awk '$2!="" {print $2; exit}')
  if [[ -n "$MOUNTED_PATH" ]]; then
    TARGET_DIR="$MOUNTED_PATH"
  else
    i=0
    while true; do
      MNT_PATH="/mnt/rpi_target$i"
      if ! mountpoint -q "$MNT_PATH"; then break; fi
      ((i++))
    done
    mkdir -p "$MNT_PATH"
    if mount "$DEV_NAME" "$MNT_PATH"; then
      TARGET_DIR="$MNT_PATH"
      TEMP_MOUNTED=true
    else
      echo "❌ Не удалось примонтировать."
      exit 1
    fi
  fi
fi
# --- Выбор устройства / конец блока ---

# 📂 Поиск образов
mapfile -t IMG_LIST < <(find "$TARGET_DIR" -maxdepth 1 -type f \( -name "*.img" -o -name "*.img.gz" \))
if [[ ${#IMG_LIST[@]} -eq 0 ]]; then
  echo "❌ Не найдено .img файлов в $TARGET_DIR"
  exit 1
fi

echo "🗂️ Найденные образы:"
i=1
for img in "${IMG_LIST[@]}"; do
  echo "  $i - $img"
  ((i++))
done

echo "  0 - ❌ Отмена"
read -rp "👉 Выберите образ: " img_choice
[[ "$img_choice" == "0" || -z "$img_choice" ]] && echo "❎ Отменено." && exit 0
(( img_choice < 1 || img_choice > ${#IMG_LIST[@]} )) && echo "❌ Неверный выбор." && exit 1
IMG_FILE="${IMG_LIST[$((img_choice-1))]}"

# 🗜 Распаковка .gz
if [[ "$IMG_FILE" =~ \.gz$ ]]; then
  echo "🗜 Найден архив. Распаковать .gz в .img?"
  read -rp "(y/N): " unpack_confirm
  if [[ "$unpack_confirm" =~ ^[Yy]$ ]]; then
    gunzip -k "$IMG_FILE" || { echo "❌ Не удалось распаковать."; exit 1; }
    IMG_FILE="${IMG_FILE%.gz}"
    echo "✅ Распаковано: $IMG_FILE"
  else
    echo "❎ Работа возможна только с .img-файлом. Завершение."
    exit 0
  fi
fi

# Подключаем образ
LOOPDEV=$(losetup --show -Pf "$IMG_FILE")
sleep 0.5
ROOT_PART=$(ls "${LOOPDEV}"p* | grep -E 'p2$' || true)

if [[ -z "$ROOT_PART" || ! -b "$ROOT_PART" ]]; then
  echo "[!] Не удалось найти rootfs-раздел (p2) в $IMG_FILE"
  exit 0
fi

# Точка монтирования rootfs
MOUNT_DIR="/mnt/rpi_seal_embed"
mkdir -p "$MOUNT_DIR"
mount "$ROOT_PART" "$MOUNT_DIR"
echo "✅ Раздел rootfs смонтирован: $MOUNT_DIR"

# Получение пользователей с UID >= 1000
mapfile -t USERS < <(awk -F: '$3 >= 1000 && $1 != "nobody" { print $1 }' "$MOUNT_DIR/etc/passwd")
UPDATED_COUNT=0

for USER in "${USERS[@]}"; do
  echo "[debug] Проверяю пользователя $USER..."
  USER_HOME="$MOUNT_DIR/home/$USER"
  BASHRC="$USER_HOME/.bashrc"
  ALIASES="$USER_HOME/.bash_aliases"

  FOUND_ALIAS=""
  if grep -q "rpi_seal" "$BASHRC" 2>/dev/null; then
    FOUND_ALIAS=$(grep "alias rpi_seal=" "$BASHRC" | head -n1)
  elif grep -q "rpi_seal" "$ALIASES" 2>/dev/null; then
    FOUND_ALIAS=$(grep "alias rpi_seal=" "$ALIASES" | head -n1)
  fi

  if [[ -n "$FOUND_ALIAS" ]]; then
    echo "[debug] Найден alias: $FOUND_ALIAS"
    SEAL_PATH=$(echo "$FOUND_ALIAS" | grep -oE 'bash +[^ ]+/rpi_seal' | sed 's/bash *//')
    SEAL_PATH=${SEAL_PATH/#\~//home/$USER}

    TARGET_PATH="$MOUNT_DIR$SEAL_PATH"
    echo "[debug] Проверяю путь: $TARGET_PATH"

    if [[ -d "$TARGET_PATH" ]]; then
      echo "[*] Обновляю rpi_seal для пользователя $USER → $SEAL_PATH"
      shopt -s dotglob nullglob
      cp -r "$LOCAL_SEAL_DIR"/* "$TARGET_PATH"
      shopt -u dotglob nullglob
      chown -R "$USER:$USER" "$TARGET_PATH" 2>/dev/null || echo "[!] Не удалось изменить владельца для $TARGET_PATH"
      ((UPDATED_COUNT++))
      echo "✅ Успешно обновлено для пользователя $USER → $SEAL_PATH"
    else
      echo "[!] Путь $SEAL_PATH не существует в образе. Пропускаю."
    fi
  else
    echo "[-] У пользователя $USER не найден alias rpi_seal. Пропускаю."
  fi
  echo

done

echo "[debug] Обновление завершено, всего обновлено: $UPDATED_COUNT"
if [[ $UPDATED_COUNT -gt 0 ]]; then
  echo "✅ Обновление rpi_seal выполнено для $UPDATED_COUNT пользователь(а/ей)."
  echo "✨ rpi_seal успешно внедрён в выбранный образ."
else
  echo "ℹ️ Ни один alias rpi_seal не был найден. Обновление не потребовалось."
fi

read -rp "⏎ Завершено. Нажмите Enter..." dummy || true

echo "✅ Всё готово. Образ размонтирован, файл доступен на носителе. Его можно архивировать или извлечь диск."
exit 0
