#!/bin/bash
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8
# ==DIS/AUTOEXEC==
# === Копировать .img ===
# копирование оригинального образа для тестов

src_dir="/mnt/backup/original"
dst_dir="/mnt/backup"

img_list=($(find "$src_dir" -maxdepth 1 \( -name "*.img" -o -name "*.img.zst" -o -name "*.img.gz" \) | sort))
count=${#img_list[@]}

if [[ $count -eq 0 ]]; then
  echo "❌ Нет файлов .img(.zst/.gz) в $src_dir"
  exit 1
fi

echo "📦 Доступные образы и архивы:" 
for i in "${!img_list[@]}"; do
  size=$(du -h "${img_list[$i]}" | cut -f1)
  echo "$((i+1)) - $(basename "${img_list[$i]}") [$size]"
done
echo "0  - 🔙 Назад"
echo "00 - ❌ Отмена"

read -p "Выберите файл для копирования/распаковки: " choice

if [[ "$choice" == "00" ]]; then
  echo "❌ Отмена"
  exit 1
elif [[ "$choice" == "0" ]]; then
  echo "🔙 Возврат в меню"
  exit 0
elif ! [[ "$choice" =~ ^[0-9]+$ ]] || (( choice < 1 || choice > count )); then
  echo "⚠️ Неверный выбор"
  exit 1
fi

selected="${img_list[$((choice-1))]}"
echo "✅ Вы выбрали: $(basename "$selected")"

read -p "Подтвердить операцию? (y/д): " confirm
confirm=$(echo "$confirm" | tr '[:upper:]' '[:lower:]')

if [[ "$confirm" != "y" && "$confirm" != "д" ]]; then
  echo "❌ Операция отменена"
  exit 1
fi

filename=$(basename "$selected")
new_name="$(date +%Y%m%d-%H%M%S).img"
dst_path="$dst_dir/$new_name"

# Проверка наличия утилит
for cmd in pv zstd gzip; do
  if ! command -v "$cmd" &>/dev/null; then
    echo "⚠️ Утилита '$cmd' не найдена. Установите её: sudo apt install $cmd"
    exit 1
  fi
done

case "$filename" in
  *.img)
    echo "🚀 Копирование с прогрессом..."
    pv "$selected" > "$dst_path"
    ;;
  *.img.zst)
    echo "📦 Распаковка .zst в $dst_path..."
    zstd -d -c "$selected" | pv > "$dst_path"
    ;;
  *.img.gz)
    echo "📂 Что сделать с архивом .gz:"
    echo "  1 - Только распаковать в .img"
    echo "  2 - Распаковать и пересжать в .zst"
    echo "  0 - Отмена"
    read -rp "👉 Ваш выбор: " action

    case "$action" in
      1)
        echo "📦 Распаковка .gz в $dst_path..."
        gunzip -c "$selected" | pv --force > "$dst_path"
        ;;
      2)
        echo "📦 Распаковка .gz и пересжатие в .zst..."
        gunzip -c "$selected" | pv --force | zstd -19 -T0 -o "$dst_path.zst"
        dst_path+=".zst"
        ;;
      *)
        echo "❌ Операция отменена"
        exit 0
        ;;
    esac
    ;;
  *)
    echo "❌ Неподдерживаемый формат: $filename"
    exit 1
    ;;
esac

echo "🎉 Готово:"
echo "   $selected → $dst_path"