#!/bin/bash
export LANG=en_US.UTF-8
export LC_ALL=en_US.UTF-8
# ==DIS/AUTOEXEC==
# pishrink_limited.sh — shrink с ограничением по минимальному размеру (в МБ)
#

set -euo pipefail

MAX_MB=0
POSITIONAL=()

while [[ $# -gt 0 ]]; do
  case "$1" in
    --max-size-mb)
      MAX_MB="$2"
      shift 2
      ;;
    *)
      POSITIONAL+=("$1")
      shift
      ;;
  esac
done

set -- "${POSITIONAL[@]}"
IMG="$1"

if [[ -z "$IMG" || ! -f "$IMG" ]]; then
  echo "❌ Укажи .img-файл"
  exit 1
fi

echo "[*] Подключаю $IMG..."
LOOP_DEV=$(sudo losetup --show -Pf "$IMG")
ROOT_PART="${LOOP_DEV}p2"

echo "[*] Проверка файловой системы..."
if ! sudo e2fsck -f -y "$ROOT_PART"; then
  echo "⚠️ e2fsck завершился с ошибками, но FS исправлена — продолжаем"
fi

echo "[*] Получаю минимальный размер rootfs..."
BLOCKS=$(sudo resize2fs -P "$ROOT_PART" | awk '{print $NF}')
BLOCK_SIZE=4096
USED_MB=$((BLOCKS * BLOCK_SIZE / 1024 / 1024))

echo "📏 Минимальный размер rootfs: $USED_MB МБ"
# ⛔ Ограничение: если shrink может сделать меньше, чем лимит — остановим на лимите
# Пример: resize2fs хочет 15000 МБ, а нам нужно не меньше 20000 → сделаем 20000
if [[ "$MAX_MB" -gt 0 && "$USED_MB" -lt "$MAX_MB" ]]; then
  echo "⚠️ Используем лимит: $MAX_MB МБ"
  BLOCKS=$((MAX_MB * 1024 * 1024 / BLOCK_SIZE))
fi

echo "[*] Уменьшаю раздел до $BLOCKS блоков..."
sudo resize2fs "$ROOT_PART" $BLOCKS
sleep 1

# Надёжное определение конца второго раздела (ext4)
END_SECTOR=$(sudo parted "$LOOP_DEV" -s unit s print | awk '/^ [0-9]+/ && /ext4/ { gsub("s", "", $3); print $3 }' | tail -n1)
SECTOR_SIZE=$(sudo blockdev --getss "$LOOP_DEV")
NEW_SIZE=$(( (END_SECTOR + 1) * SECTOR_SIZE ))

echo "[*] Отмонтирую..."
sudo losetup -d "$LOOP_DEV"

echo "[*] Обрезаю образ до $((NEW_SIZE / 1024 / 1024)) МБ..."
truncate -s "$NEW_SIZE" "$IMG"

echo "✅ Готово! Новый размер: $((NEW_SIZE / 1024 / 1024)) МБ (лимит: ${MAX_MB:-0} МБ)"
